import os
import struct
import select
from collections import deque
import platform


#import amp
import libamplifier_lib as amp
from pylogger import bllogger


class KeyHandler:
    
    def __init__(self, amp_settings):
        self.amp_settings = amp_settings

        self.gpio_fd = None
                
        self.error_hist = deque(maxlen=100)

        self.command_dict = {
                amp.key_map.LEFT_INPUT_DC: {},
                amp.key_map.RIGHT_INPUT_DC: {},
                amp.key_map.LEFT_CLIPPING: {},
                amp.key_map.RIGHT_CLIPPING: {},
                amp.key_map.LEFT_OUTPUT_DC: {},
                amp.key_map.RIGHT_OUTPUT_DC: {},
                amp.key_map.LEFT_THERMAL_ERROR: {},
                amp.key_map.RIGHT_THERMAL_ERROR: {},
                amp.key_map.LEFT_NEG_15VOLT_ERROR:{},
                amp.key_map.RIGHT_NEG_15VOLT_ERROR: {},
                amp.key_map.LEFT_POS_15VOLT_ERROR: {},
                amp.key_map.RIGHT_POS_15VOLT_ERROR: {},
                amp.key_map.LEFT_NEG_IV_ERROR: {},
                amp.key_map.RIGHT_NEG_IV_ERROR: {},
                amp.key_map.LEFT_POS_IV_ERROR: {},
                amp.key_map.RIGHT_POS_IV_ERROR: {},
                amp.key_map.TRIG_12V: {},
                amp.key_map.POWER_ON: {},
                amp.key_map.DIAG1: {},
                amp.key_map.DIAG2: {},
                amp.key_map.DIAG3: {}
                } 

        self.settings_dict = {
                amp.key_map.LEFT_2100_1: {},
                amp.key_map.LEFT_2100_2: {},
                amp.key_map.RIGHT_2100_1: {},
                amp.key_map.RIGHT_2100_2: {},
                amp.key_map.LEFT_MONO_PLUS: {},
                amp.key_map.RIGHT_MONO_PLUS: {}
                }

        self.diag_gpios = {
                amp.key_map.DIAG1: 30,
                amp.key_map.DIAG2: 2,
                amp.key_map.DIAG3: 3
                }


    def get_gpio_fd(self):
        self.gpio_fd = []
        try:
            if 'arm'  in platform.machine():
                device_string = 'gpio_keys'
            else:
                device_string = 'kbd'
            for device in os.listdir('/dev/input/by-path'):
                if device_string in device:	
                    self.gpio_fd.append(os.open(os.path.join('/dev/input/by-path', device),os.O_RDONLY))
                    break
        except:
            print "could not open folder ", os.path.join('/dev/input/by-path', device)

        try:
            self.gpio_fd.append( os.open(os.path.join('/dev/input', 'event1'),os.O_RDONLY))
        except:
            print "Keyboard not found, will not it to gpio_fd"

        if self.gpio_fd is None:
                raise "Error could not find the keybaord"
                return


    def poll(self):
        self.poller = select.poll()
        for fd in self.gpio_fd:
            self.poller.register(fd, select.POLLIN)
        EVENT_FORMAT = 'llHHI'
        EVENT_SIZE = struct.calcsize(EVENT_FORMAT)
        keys = []

        event = [1,1,1,1,1]
        if not hasattr(self, 'poller'):
            return [-1, -1]
        try:
            response = self.poller.poll(500)
        except select.error as ex:
            if ex[0] == 4:
                #ignore interrupted system call errors
                response = []
            else:
                raise

        for gpio_fd, in_out in response:
            while event[3] != 0 :
                try:
                    gpio_event =  os.read(gpio_fd, EVENT_SIZE)
                except:
                    print "ERROR no file descriptor"
                    break
                event = struct.unpack(EVENT_FORMAT, gpio_event)
                if event[2] == 1:
                    keys.append(event[3])
                    keys.append(event[4])
                    return keys
        #print map(hex, keys)
        return [-1,-1]

    def process(self):
        key = 0
        while key is not -1:
            key, pressed = self.poll()
            pressed = bool(pressed)
            if key in self.command_dict:
                self.amp_settings.add_error(key, pressed)
            elif key != -1:
                bllogger('err', 'Key pressed  not in command_dict: %d' % key)

            #do a seperate check for diag pins (need to check level not edge)
            for key, pressed in self.get_diag_states():
                self.amp_settings.add_error(key, pressed)
                key = -1

           

    def check_diag(self):
        key = 0
        while key is not -1:
            for key, pressed in self.get_diag_states():
                if key == amp.key_map.DIAG3 and  pressed:
                    self.amp_settings.add_error(key, pressed)
                    return True
        return False 

    def get_diag_states(self):
        diag_states = []
        for gpio in self.diag_gpios:
            fname = os.path.join("/sys","class","gpio","gpio%d"%self.diag_gpios[gpio],"value")
            with open(fname, 'r') as fp:
                state = int(fp.readline().strip())
            if self.amp_settings.command_dict[gpio]['get']() != state:
                diag_states.append((gpio, state))
        return diag_states

            
    def check_abort(self):
        """
        Used durring power up procedure, checks if the amp should abort power up
        """
        key = 0
        while key is not -1:
            key, pressed = self.poll()
            pressed = bool(pressed)
            if key == amp.key_map.POWER_ON:
                if pressed:
                    return True
            elif key == amp.key_map.TRIG_12V:
                if not pressed:
                    return True
        return False
